@@ -1,5 +1,6 @@
 //! Module for items related to aggregating validation_receipts
 
+use futures::StreamExt;
 use holo_hash::AgentPubKey;
 use holo_hash::DhtOpHash;
 use holochain_keystore::AgentPubKeyExt;
@@ -39,7 +40,7 @@ pub struct ValidationReceipt {
     pub validation_status: ValidationStatus,
 
     /// the remote validator which is signing this receipt.
-    pub validator: AgentPubKey,
+    pub validators: Vec<AgentPubKey>,
 
     /// Time when the op was integrated
     pub when_integrated: Timestamp,
@@ -51,10 +52,36 @@ impl ValidationReceipt {
         self,
         keystore: &MetaLairClient,
     ) -> holochain_keystore::LairResult<SignedValidationReceipt> {
-        let signature = self.validator.sign(keystore, self.clone()).await?;
+        let this = self.clone();
+        // Try to sign with all validators but silently fail on
+        // any that cannot sign.
+        // If all signatures fail then return an error.
+        let futures = self
+            .validators
+            .iter()
+            .map(|validator| {
+                let this = this.clone();
+                let validator = validator.clone();
+                let keystore = keystore.clone();
+                async move { validator.sign(&keystore, this).await }
+            })
+            .collect::<Vec<_>>();
+        let signatures = futures::stream::iter(futures)
+            .buffer_unordered(10)
+            .collect::<Vec<_>>()
+            .await;
+        let signatures = if signatures.iter().any(Result::is_ok) {
+            signatures.into_iter().filter_map(Result::ok).collect()
+        } else {
+            return Err(signatures
+                .into_iter()
+                .filter_map(Result::err)
+                .next()
+                .expect("Must contain at least one error if there is no success"));
+        };
         Ok(SignedValidationReceipt {
             receipt: self,
-            validator_signature: signature,
+            validators_signatures: signatures,
         })
     }
 }
@@ -77,7 +104,7 @@ pub struct SignedValidationReceipt {
     pub receipt: ValidationReceipt,
 
     /// the signature of the remote validator.
-    pub validator_signature: Signature,
+    pub validators_signatures: Vec<Signature>,
 }
 
 pub fn list_receipts(
@@ -136,7 +163,7 @@ mod tests {
         let receipt = ValidationReceipt {
             dht_op_hash: dht_op_hash.clone(),
             validation_status: ValidationStatus::Valid,
-            validator: agent,
+            validators: vec![agent],
             when_integrated: Timestamp::now(),
         };
         receipt.sign(keystore).await.unwrap()
@@ -146,7 +173,7 @@ mod tests {
     async fn test_validation_receipts_db_populate_and_list() -> StateMutationResult<()> {
         observability::test_run().ok();
 
-        let test_env = crate::test_utils::test_cell_env();
+        let test_env = crate::test_utils::test_authored_env();
         let env = test_env.env();
         let keystore = crate::test_utils::test_keystore();
 
@@ -157,7 +184,7 @@ mod tests {
         let test_op_hash = op.as_hash().clone();
         env.conn()
             .unwrap()
-            .with_commit_sync(|txn| mutations::insert_op(txn, op, true))
+            .with_commit_sync(|txn| mutations::insert_op(txn, op))
             .unwrap();
 
         let vr1 = fake_vr(&test_op_hash, &keystore).await;
@@ -181,17 +208,15 @@ mod tests {
 
             let mut list = list_receipts(&reader, &test_op_hash).unwrap();
             list.sort_by(|a, b| {
-                a.receipt
-                    .validator
-                    .partial_cmp(&b.receipt.validator)
+                a.receipt.validators[0]
+                    .partial_cmp(&b.receipt.validators[0])
                     .unwrap()
             });
 
             let mut expects = vec![vr1, vr2];
             expects.sort_by(|a, b| {
-                a.receipt
-                    .validator
-                    .partial_cmp(&b.receipt.validator)
+                a.receipt.validators[0]
+                    .partial_cmp(&b.receipt.validators[0])
                     .unwrap()
             });
 
